package edu.uiuc.ncsa.security.servlet;

import edu.uiuc.ncsa.security.core.configuration.Configurations;
import edu.uiuc.ncsa.security.core.exceptions.ConfigurationException;
import org.apache.commons.configuration.XMLConfiguration;
import org.apache.commons.configuration.tree.ConfigurationNode;

import javax.servlet.ServletContext;
import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;

/**
 * Static utilities for working with configurations and servlets.
 * <p>Created by Jeff Gaynor<br>
 * on 3/23/12 at  8:23 AM
 */
public class ConfigUtil {
    /**
     * Finds the configuration in an XML file, given the filename, name of the configuration
     * (or null if you want to use the only one there) and the top node tag
     * (E.g. "client" or "server").
     * @param fileName
     * @param configName
     * @param topNodeTag
     * @return
     */
    public static ConfigurationNode findConfiguration(String fileName,
                                                      String configName,
                                                      String topNodeTag) {
        if (fileName == null || fileName.length() == 0) {
            throw new ConfigurationException("Error: No configuration file specified");
        }
        XMLConfiguration cfg = null;
        // A properties file is supplied. Use that.
        File file = new File(fileName);
        if (!(file.exists() && file.isFile())) {
            throw new ConfigurationException("Error: file \"" + file + "\" does not exist");
        }
        cfg = Configurations.getConfiguration(new File(fileName));
        return findNamedConfig(cfg, configName, topNodeTag);

    }

    /**
     * Looks for the configuration in the servlet context using the keys for the init parameters
     * @param servletContext
     * @param configFileKey
     * @param configName
     * @param topNodeTag
     * @return
     */

    public static ConfigurationNode findConfigurationNode(ServletContext servletContext,
                                                          String configFileKey,
                                                          String configName,
                                                          String topNodeTag) {
        String fileName = servletContext.getInitParameter(configFileKey);
        if(fileName == null || fileName.length() == 0){
            throw new ConfigurationException("Error: No configuration file was specified in the servlet configuration.");
        }
        try {
            return findConfiguration(fileName, servletContext.getInitParameter(configName), topNodeTag);
        } catch (ConfigurationException cx) {
            cx.printStackTrace();
            // plan B, maybe it's in the deployment itself? try to get as a resource
            URL url = null;
            try {
                url = servletContext.getResource(fileName);
            } catch (MalformedURLException e) {
                throw new ConfigurationException("Error: Could not parse URL \"" + fileName + "\". Has a valid configuration been specified?", e);
            }
            if (url == null) {
                throw new ConfigurationException("Error:No configuration found.");
            }
            XMLConfiguration cfg = Configurations.getConfiguration(url);
            return findNamedConfig(cfg, servletContext.getInitParameter(configName), topNodeTag);
        }
    }

    public static ConfigurationNode findNamedConfig(XMLConfiguration cfg, String cfgName, String topNodeTag) {
        ConfigurationNode cn = null;
        if (cfgName == null) {
            cn = cfg.configurationAt(topNodeTag).getRootNode();
        } else {
            cn = Configurations.getConfig(cfg, topNodeTag, cfgName);
        }

        return cn;
    }


}
